==> reverie_dreams-0.4.3%2B1.21.7%2Bbeta-sources.jar: cc/thonly/reverie_dreams/item/prop/DreamPillowItem.java <==
package cc.thonly.reverie_dreams.item.prop;

import cc.thonly.reverie_dreams.interfaces.IDreamPillowManager;
import cc.thonly.reverie_dreams.server.DreamPillowManager;
import net.minecraft.class_1269;
import net.minecraft.class_1657;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1838;
import net.minecraft.class_1937;
import net.minecraft.class_2244;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2383;
import net.minecraft.class_2680;
import net.minecraft.class_2742;
import net.minecraft.class_3218;
import net.minecraft.class_3222;
import net.minecraft.class_3545;
import net.minecraft.server.MinecraftServer;

public class DreamPillowItem extends class_1792 {

    public DreamPillowItem(class_1793 settings) {
        super(settings);
    }

    @Override
    public class_1269 method_7884(class_1838 context) {
        class_1657 player = context.method_8036();
        class_1937 world = context.method_8045();
        if (!world.field_9236 && world instanceof class_3218 serverWorld && player instanceof class_3222) {
            boolean sneaking = player.method_5715();
            class_1799 itemStack = player.method_5998(context.method_20287());
            MinecraftServer server = serverWorld.method_8503();
            if (sneaking) {
                IDreamPillowManager iDreamPillowManager = (IDreamPillowManager) server;
                DreamPillowManager dreamPillowManager = iDreamPillowManager.getDreamPillowManager();
                DreamPillowManager.WorldEntry worldEntry = dreamPillowManager.get(serverWorld);
                class_2338 blockPos = context.method_8037();
                class_3545<Boolean, class_2338> bedHead = getBedHead(serverWorld, blockPos);
                if (!worldEntry.contains(bedHead.method_15441()) && bedHead.method_15442()) {
                    worldEntry.add(blockPos);
                    itemStack.method_57008(1, player);
                    return class_1269.field_52422;
                }
            }
            return class_1269.field_5814;
        }
        return class_1269.field_5812;
    }

    public static class_3545<Boolean, class_2338> getBedHead(class_3218 serverWorld, class_2338 blockPos) {
        class_2680 blockState = serverWorld.method_8320(blockPos);
        if (blockState.method_26204() instanceof class_2244) {
            class_2742 bedPart = blockState.method_11654(class_2244.field_9967);
            class_2350 direction = blockState.method_11654(class_2383.field_11177);
            class_2338 headPos;

            if (bedPart == class_2742.field_12560) {
                headPos = blockPos;
            } else {
                headPos = blockPos.method_10093(direction);
            }
            return new class_3545<>(true, headPos);
        }
        return new class_3545<>(false, null);
    }

}

==> reverie_dreams-0.4.7%2B1.21.7%2Bbeta-sources.jar: cc/thonly/reverie_dreams/item/prop/DreamPillowItem.java <==
package cc.thonly.reverie_dreams.item.prop;

import cc.thonly.reverie_dreams.interfaces.IBedBlockEntity;
import net.minecraft.class_1269;
import net.minecraft.class_1657;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1838;
import net.minecraft.class_1937;
import net.minecraft.class_2244;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2383;
import net.minecraft.class_2587;
import net.minecraft.class_2680;
import net.minecraft.class_2742;
import net.minecraft.class_3218;
import net.minecraft.class_3222;
import net.minecraft.class_3545;
import net.minecraft.server.MinecraftServer;

public class DreamPillowItem extends class_1792 {

    public DreamPillowItem(class_1793 settings) {
        super(settings);
    }

    @Override
    public class_1269 method_7884(class_1838 context) {
        class_1657 player = context.method_8036();
        class_1937 world = context.method_8045();
        if (!world.field_9236 && world instanceof class_3218 serverWorld && player instanceof class_3222) {
            boolean sneaking = player.method_5715();
            class_1799 itemStack = player.method_5998(context.method_20287());
            class_2338 blockPos = context.method_8037();
            class_3545<Boolean, class_2338> bedHead = getBedHead(serverWorld, blockPos);
            if (sneaking && bedHead.method_15442() && serverWorld.method_8321(bedHead.method_15441()) instanceof class_2587 blockEntity) {
                IBedBlockEntity iBedBlockEntity = (IBedBlockEntity) blockEntity;
                if (iBedBlockEntity.hasDreamPillow()) {
                    return class_1269.field_5811;
                } else {
                    iBedBlockEntity.setHasDreamPillow(true);
                    itemStack.method_57008(1, player);
                    return class_1269.field_52422;
                }

            }
            return class_1269.field_5814;
        }
        return class_1269.field_5812;
    }

    public static class_3545<Boolean, class_2338> getBedHead(class_3218 serverWorld, class_2338 blockPos) {
        class_2680 blockState = serverWorld.method_8320(blockPos);
        if (blockState.method_26204() instanceof class_2244) {
            class_2742 bedPart = blockState.method_11654(class_2244.field_9967);
            class_2350 direction = blockState.method_11654(class_2383.field_11177);
            class_2338 headPos;

            if (bedPart == class_2742.field_12560) {
                headPos = blockPos;
            } else {
                headPos = blockPos.method_10093(direction);
            }
            return new class_3545<>(true, headPos);
        }
        return new class_3545<>(false, null);
    }

}
